@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2018 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -725,6 +725,45 @@ public:
     void evaluate(Pkt& pkt, ValueStack& values);
 };
 
+/// @brief Token that converts to hexadecimal string
+///
+/// For example in the sub-expression "hexstring(pkt4.mac, ':')"
+/// the binary MAC address is converted to its usual hexadecimal
+/// representation as a list of (6) pairs of hexadecimal digits
+/// separated by colons (':').
+/// Please note the token is named TokenToHexString when the syntax
+/// use the hexstring name without a leading "to".
+class TokenToHexString : public Token {
+public:
+    /// @brief Constructor (does nothing)
+    TokenToHexString() { }
+
+    /// @brief Convert a binary value to its hexadecimal string representation
+    ///
+    /// Evaluation does not use packet information. It requires at least
+    /// two values to be present on the stack. It will consume the top
+    /// two values on the stack as parameters and push the resulting
+    /// hexadecimal string onto the stack.
+    /// From the top it expects the values on the stack as:
+    /// - separator
+    /// - binary
+    ///
+    /// binary is the binary value (note it can be any value, i.e.
+    /// it is not checked to really be not printable).
+    /// separator is litteral for instance '-' or ':'. The empty separator
+    /// means no separator.
+    ///
+    /// The following example use a binary MAC address 06:ce:8f:55:b3:33:
+    /// - mac, '-' => "06-ce-8f-55-b3-33"
+    ///
+    /// @throw EvalBadStack if there are less than 2 values on stack
+    ///
+    /// @param pkt (unused)
+    /// @param values - stack of values (2 arguments will be popped, 1 result
+    ///        will be pushed)
+    void evaluate(Pkt& pkt, ValueStack& values);
+};
+
 /// @brief Token that represents logical negation operator
 ///
 /// For example in the expression "not(option[vendor-class].text == 'MSF')"
@@ -800,6 +839,40 @@ public:
     void evaluate(Pkt& pkt, ValueStack& values);
 };
 
+/// @brief Token that represents client class membership
+///
+/// For example "not member('foo')" is the complement of class foo
+class TokenMember : public Token {
+public:
+    /// @brief Constructor
+    ///
+    /// @param client_class client class name
+    TokenMember(const std::string& client_class)
+        :client_class_(client_class){
+    }
+
+    /// @brief Token evaluation (check if client_class_ was added to
+    /// packet client classes)
+    ///
+    /// @param pkt the class name will be check from this packet's client classes
+    /// @param values true (if found) or false (if not found) will be pushed here
+    void evaluate(Pkt& pkt, ValueStack& values);
+
+    /// @brief Returns client class name
+    ///
+    /// This method is used in testing to determine if the parser had
+    /// instantiated TokenMember with correct parameters.
+    ///
+    /// @return client class name the token expects to check membership.
+    const ClientClass& getClientClass() const {
+        return (client_class_);
+    }
+
+protected:
+    /// @brief The client class name
+    ClientClass client_class_;
+};
+
 /// @brief Token that represents vendor options in DHCPv4 and DHCPv6.
 ///
 /// It covers vendor independent vendor information option (125, DHCPv4)
